/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package swinggrapheditor.graph;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

/**
 *
 * @author trust56
 */
public class LocatedGraph<T extends Serializable> extends Graph<T, LocatedNode<T>> implements Serializable {

    protected final int NODE_RADIUS = 10;
    protected LocatedNode<T> selectedNode;
    protected Arc<T, LocatedNode<T>> selectedArc;

    public static boolean pointsCloserThan(double distance,
            Point point1, Point point2) {
        if (point1 == null || point2 == null) {
            return false;
        } else {
            int dx = Math.abs(point1.x - point2.x);
            int dy = Math.abs(point1.y - point2.y);
            return distance * distance > dx * dx + dy * dy;
        }
    }

    public void addArc(Point point) {
        if (selectedNode != null) {
            Arc<T, LocatedNode<T>> arc = new Arc<>();
            arc.edge1 = selectedNode;
            for (LocatedNode<T> locatedNode : nodes) {
                if (pointsCloserThan(NODE_RADIUS, point, new Point(locatedNode.x, locatedNode.y))) {
                    arc.edge2 = locatedNode;
                    break;
                }
            }
            if (arc.edge2 != null) {
                arcs.add(arc);
                arc.edge1.arcs.add(arc);
                arc.edge2.arcs.add(arc);
                graphChanged();
            }
        }
    }

    public void addNode(int x, int y, T e) {
        LocatedNode<T> node = new LocatedNode<>();
        node.x = x;
        node.y = y;
        node.element = e;
        for (LocatedNode<T> locatedNode : nodes) {
            if (pointsCloserThan(NODE_RADIUS,
                    new Point(locatedNode.x, locatedNode.y), new Point(x, y))) {
                return;
            }
        }
        nodes.add(node);
        graphChanged();
    }

    public void deleteSelectedNodeOrArc() {
        if (selectedNode != null) {
            List<Arc<T, LocatedNode<T>>> arcsToDelete = new ArrayList<>();
            for (Arc<T, LocatedNode<T>> arc : arcs) {
                if (arc.edge1 == selectedNode || arc.edge2 == selectedNode) {
                    arcsToDelete.add(arc);
                }
            }
            for (Arc<T, LocatedNode<T>> arc : arcsToDelete) {
                arcs.remove(arc);
                arc.edge1.arcs.remove(arc);
                arc.edge2.arcs.remove(arc);
            }
            nodes.remove(selectedNode);
            selectedNode = null;
            graphChanged();
        }
        if (selectedArc != null) {
            selectedArc.edge1.arcs.remove(selectedArc);
            selectedArc.edge2.arcs.remove(selectedArc);
            arcs.remove(selectedArc);
            graphChanged();
        }
    }

    public void dragPoint(Point point) {
        if (selectedNode != null) {
            selectedNode.x = point.x;
            selectedNode.y = point.y;
            graphChanged();
        }
    }
    private List<GraphListener> graphListeners = new ArrayList<>();

    protected void graphChanged() {
        for (GraphListener graphListener : graphListeners) {
            graphListener.graphChanged();
        }
    }

    public void addGraphListener(GraphListener graphListener) {
        graphListeners.add(graphListener);
    }

    public void removeGraphListener(GraphListener graphListener) {
        graphListeners.remove(graphListener);
    }

    public void selectPoint(Point point) {
        LocatedNode<T> oldSelectedNode = selectedNode;
        Arc<T, LocatedNode<T>> oldSelectedArc = selectedArc;
        selectedNode = null;
        selectedArc = null;
        for (Arc<T, LocatedNode<T>> arc : arcs) {
            double tangent1 = (arc.edge1.x - arc.edge2.x)
                    / (double) (arc.edge1.y - arc.edge2.y);
            double tangent2 = (point.x - arc.edge2.x)
                    / (double) (point.y - arc.edge2.y);
            double tangent3 = (point.x - arc.edge1.x)
                    / (double) (point.y - arc.edge1.y);
            boolean ok = Math.abs(tangent1 - tangent2) < 0.2 && Math.abs(tangent1 - tangent3) < 0.2;
            //ok = ok && (arc.edge1.x > arc.edge2.x) ? arc.edge1.x > point.x : arc.edge1.x < point.x;
            if (ok) {
                selectedArc = arc;
            }
        }
        for (LocatedNode<T> locatedNode : nodes) {
            if (pointsCloserThan(NODE_RADIUS,
                    point, new Point(locatedNode.x, locatedNode.y))) {
                selectedNode = locatedNode;
                selectedArc = null;
            }
        }
        if (selectedNode != oldSelectedNode
                || selectedArc != oldSelectedArc) {
            graphChanged();
        }
    }

    public void setElementInSelectedNode(T element) {
        if (selectedNode != null) {
            selectedNode.element = element;
            graphChanged();
        }
    }
//    @Override
//    public LocatedGraph<T> clone() throws CloneNotSupportedException {
//        LocatedGraph<T> clone = new LocatedGraph<>();
//        clone.arcs = this.arcs;
//        clone.nodes = this.nodes;
//        clone.selectedArc = this.selectedArc;
//        clone.selectedNode = this.selectedNode;
//        return clone;
//    }
}
