package swinggrapheditor.xmlutils;

import java.io.StringWriter;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;
import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class XmlMaker {

    private final Document document;
    private Element actElement;
    
    public XmlMaker(String rootTagName) throws ParserConfigurationException {
        document = DocumentBuilderFactory.newInstance().newDocumentBuilder().newDocument();
        document.setXmlStandalone(true);
        Element rootElement = document.createElement(rootTagName);
        document.appendChild(rootElement);
        actElement = rootElement;
    }    
    
    public String getXmlAsString() throws TransformerConfigurationException, TransformerException{
        StringWriter sw = new StringWriter();
        final Transformer transformer = TransformerFactory.newInstance().newTransformer();
        transformer.setOutputProperty("indent", "yes");
        transformer.transform(new DOMSource(document), new StreamResult(sw));
        return sw.toString();
    }

    public void insertElement(String elementName) {
        Element newElement = document.createElement(elementName);
        actElement.appendChild(newElement);
        actElement = newElement;
    }

    public void addAttribute(String attributeName, String attributeValue) {
        actElement.setAttribute(attributeName, attributeValue);
    }

    public void stepOut() {
        actElement = (Element) actElement.getParentNode();
    }

    public void insertText(String textContent) {
        actElement.setTextContent(textContent);
    }
}
